
/******************************************************************************

Coral Tree nl/mplex.cc
Copyright (C) 1998 Infinite Monkeys Incorporated

This program is free software; you can redistribute it and/or modify
it under the terms of the Artistic License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
Artistic License for more details.

You should have received a copy of the Artistic License along with
this program (see meta/ART_LICE); if not, write to coral@imonk.com.

******************************************************************************/

#include <nl/nl.h>

long NL_ChannelNode::count = 0;


/******************************************************************************
NL_Multiplexor::NL_Multiplexor(void)

******************************************************************************/
NL_Multiplexor::NL_Multiplexor(void)
{
	msec_timeout = 0;

	select_timeout.tv_sec = 0;
	select_timeout.tv_usec = 0;

	FD_ZERO(&select_readfds);
	FD_ZERO(&select_writefds);
	FD_ZERO(&select_exceptfds);
}

/******************************************************************************
NL_Multiplexor::~NL_Multiplexor(void)

Clean out both the check list and the ready list.
******************************************************************************/
NL_Multiplexor::~NL_Multiplexor(void)
{
	// Clean out linked lists.
	register NL_ChannelNode *channel_node;
	readylist.ToHead();
	while (NULL != (channel_node = readylist.PostIncrement()))
	{
		readylist.RemoveNode(channel_node);
		delete channel_node;
	}

	checklist.ToHead();
	while (NULL != (channel_node = checklist.PostIncrement()))
	{
		checklist.RemoveNode(channel_node);
		delete channel_node;
	}
}

/******************************************************************************
long NL_Multiplexor::Wait(void)

TODO: implement a free list for NL_ChannelNodes instead of destroying
and reallocating them in the free list each time through.
******************************************************************************/
long NL_Multiplexor::Wait(void)
{
	long return_val = 0;

	// Clear the ready list.
	register NL_ChannelNode *channel_node;
	readylist.ToHead();
	while (NULL != (channel_node = readylist.PostIncrement()))
	{
		readylist.RemoveNode(channel_node);
		delete channel_node;
	}

	// Go through the check list.
	long do_select = FALSE;
	unsigned long ip_handle;
	int width = 0;
	struct timeval *tmp_timeout;
	checklist.ToHead();
	while (NULL != (channel_node = checklist.PostIncrement()))
	{
		// If this is an Internet domain channel...
		if (channel_node->channel->GetDomain() & NL_INET)
		{
			// Initialize the select() structures.
			if (!do_select)
			{
				do_select = TRUE;
				FD_ZERO(&select_readfds);
				FD_ZERO(&select_writefds);
				FD_ZERO(&select_exceptfds);
				if (NL_MULTIPLEXOR_BLOCK == msec_timeout)
				{
					tmp_timeout = NULL;
				}
				else
				{
					select_timeout.tv_sec = (int) (msec_timeout / 1000);
					select_timeout.tv_usec =
						1000 * (msec_timeout - (1000 * select_timeout.tv_sec));
					tmp_timeout = &select_timeout;
				}
				
			}

			// Determine the number of file descriptors to check.
			ip_handle = channel_node->channel->GetIPHandle();
			if ((unsigned long) width <= ip_handle)
			{
				width = (int) (1 + ip_handle);
			}

			// Add this channel to the file descriptor sets based on its flags.
			// See AddToCheckList().
			if (channel_node->flags & NL_READ)
			{
				FD_SET(ip_handle, &select_readfds);
			}
			if (channel_node->flags & NL_WRITE)
			{
				FD_SET(ip_handle, &select_writefds);
			}
			if (channel_node->flags & NL_EXCEPTION)
			{
				FD_SET(ip_handle, &select_exceptfds);
			}
		}
	}

	// If there are any file descriptors to check...
	if (do_select)
	{
		// Do the select().
		int n;
#if OSD_OS==OSD_HPUX && OSD_OSVER==9
    	if (0 > (n = select(width, (int *) &select_readfds,
							(int *) &select_writefds,
							(int *) &select_exceptfds, tmp_timeout)))
		{
			NL_PRINT(NL_APP, NL_ERROR, "select failed in multiplexor");
		}
#else
    	if (0 > (n = select(width, (fd_set *) &select_readfds,
							(fd_set *) &select_writefds,
							(fd_set *) &select_exceptfds, tmp_timeout)))
		{
			NL_PRINT(NL_APP, NL_ERROR, "select failed in multiplexor");
		}
#endif

		// If one or more file descriptors have activity on them...
		if (n)
		{
			// Iterate through the check list.
			register NL_ChannelNode *new_channel_node;
			long flags;
			checklist.ToHead();
			while (NULL != (channel_node = checklist.PostIncrement()))
			{
				// Determine what kind of activity is waiting for the channel.
				flags = 0;
				ip_handle = channel_node->channel->GetIPHandle();
				if (FD_ISSET(ip_handle, &select_readfds))
				{
					flags |= NL_READ;
				}
				if (FD_ISSET(ip_handle, &select_writefds))
				{
					flags |= NL_WRITE;
				}
				if (FD_ISSET(ip_handle, &select_exceptfds))
				{
					flags |= NL_EXCEPTION;
				}

				// If the channel has any activity whatsoever...
				if (flags)
				{
					// Add this channel to the ready list.
					new_channel_node =
						new NL_ChannelNode(channel_node->channel, flags);
					readylist.AppendNode(new_channel_node);
					return_val++;
				}
			}
		}
		else
		{
			/* SELECT TIMED OUT */
		}
	}

	// Return the number of channels which have activity.
	return return_val;
}

/******************************************************************************
NL_ChannelNode *NL_ChannelList::Find(NL_Channel *a_channel)

Find the node in the channel list which contains the specified channel.
Return NULL if no node contains the channel.
******************************************************************************/
NL_ChannelNode *NL_ChannelList::Find(NL_Channel *a_channel)
{
	register NL_ChannelNode *channel_node;
	ToHead();
	while (NULL != (channel_node = PostIncrement()))
	{
		if(channel_node->channel == a_channel)
		{
			return channel_node;
		}
	}
	return NULL;
}

/******************************************************************************
void NL_Multiplexor::AddToChecklist(NL_Channel *channel, long flags)

Add the channel to the check list with the given flags.
******************************************************************************/
void NL_Multiplexor::AddToChecklist(NL_Channel *channel, long flags)
{
	register NL_ChannelNode *channel_node = new NL_ChannelNode(channel, flags);
	checklist.AppendNode(channel_node);
}


