#ifndef NL_MPLEX_H
#define NL_MPLEX_H

/******************************************************************************

Coral Tree nl/mplex.h
Copyright (C) 1998 Infinite Monkeys Incorporated

This program is free software; you can redistribute it and/or modify
it under the terms of the Artistic License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
Artistic License for more details.

You should have received a copy of the Artistic License along with
this program (see meta/ART_LICE); if not, write to coral@imonk.com.

******************************************************************************/

#define NL_READ			1
#define NL_WRITE		2
#define NL_EXCEPTION	4

#define NL_MULTIPLEXOR_BLOCK	-1

/***********************************************************************
class NL_ChannelNode

***********************************************************************/
class NL_ChannelNode
{
	public:
		static long		count;
		NL_Channel		*channel;
		long			flags;

		NL_ChannelNode(NL_Channel *a_channel, long a_long)
		{
			count++;
			channel = a_channel;
			flags = a_long;
		}
		virtual ~NL_ChannelNode(void) { count--; }
};

/***********************************************************************
class NL_ChannelList

***********************************************************************/
class NL_ChannelList : public GPL_DoubleList<NL_ChannelNode>
{
	public:
		NL_ChannelList(void) {}
		virtual ~NL_ChannelList(void) {}
		NL_ChannelNode *Find(NL_Channel *a_channel);
};

/***********************************************************************
class NL_Multiplexor

***********************************************************************/
class NL_Multiplexor
{
	private:
		// Generic members.
		long			msec_timeout;
		NL_ChannelList	checklist;
		NL_ChannelList	readylist;

		// Domain-specific members.
		struct timeval	select_timeout;
		fd_set			select_readfds;
		fd_set			select_writefds;
		fd_set			select_exceptfds;

	public:
		NL_Multiplexor(void);
		~NL_Multiplexor(void);

		NL_ChannelList 	*GetChecklist(void) { return &checklist; }
		NL_ChannelList 	*GetReadylist(void) { return &readylist; }

		long	SetTimeout(long msecs)
		{
			msec_timeout = msecs;
			return TRUE;
		}
		long	GetTimeout(void) { return msec_timeout; }
		long	Wait(void);

		// Convienience function.
		void	AddToChecklist(NL_Channel *channel, long flags);
};

#endif /* NL_MPLEX_H */
