
/******************************************************************************

Coral Tree nl/system.cc
Copyright (C) 1998 Infinite Monkeys Incorporated

This program is free software; you can redistribute it and/or modify
it under the terms of the Artistic License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
Artistic License for more details.

You should have received a copy of the Artistic License along with
this program (see meta/ART_LICE); if not, write to coral@imonk.com.

******************************************************************************/

#include "nl.h"

/******************************************************************************
uint16	nlGetServicePort(char *service_name, char *protocol_name)

Return the port for a specified service and protocol.

keywords: blackbox, c, network, ip, system, service, sysinfo

******************************************************************************/
/*PROTOTYPE*/
#if OSD_COMPILER==OSD_KR
uint16 nlGetServicePort(service_name, protocol_name)
	char *service_name;
	char *protocol_name;
#else
uint16 nlGetServicePort(char *service_name, char *protocol_name)
#endif
{
	struct servent *svr_p;
	uint16 temp;

	svr_p = getservbyname(service_name, protocol_name);

	if(svr_p)
	{
		temp = ntohs((uint16) svr_p->s_port);
		return temp;
	}
	else
	{
		return 0;
	}
}


/******************************************************************************
long	nlGetServiceName(char *servicename, long servicename_length, uint16 port,
																char *protocol)

	get service name from port number

keywords: blackbox, c, network, ip, system, service, sysinfo

******************************************************************************/
/*PROTOTYPE*/
#if OSD_COMPILER==OSD_KR
long nlGetServiceName(servicename, servicename_length, port, protocol)
	char *service_name;
	long servicename_length;
	uint16 port;
	char *protocol;
#else
long nlGetServiceName(char *servicename, long servicename_length, uint16 port, char *protocol)
#endif
{
	struct servent *service_entry;

	service_entry = getservbyport(port,protocol);

	if (NULL == service_entry)
	{
		strcpy(servicename, "_unknown_");
		return FALSE;
	}
	else
	{
		strncpy(servicename, (char *) service_entry->s_name,
				(int) servicename_length);
		return TRUE;
	}
}


/******************************************************************************
int		nlGetServerHostname(char *hostname, int hostname_length,
													NL_HOST_INET_ADDR address)

Given an internet address (NL_HOST_INET_ADDR) get the hostname. Return TRUE
on success, FALSE on failure.

keywords: blackbox, c, sysinfo, system, hostname

******************************************************************************/
/*PROTOTYPE*/
#if OSD_COMPILER==OSD_KR
int nlGetServerHostname(hostname, hostname_length, address)
	char *hostname;
	int hostname_length;
	NL_HOST_INET_ADDR address;
#else
int nlGetServerHostname(char *hostname, int hostname_length, NL_HOST_INET_ADDR address)
#endif
{
	struct hostent *hostent_ptr;

#if OSD_OS==OSD_SUNOS || OSD_OS==OSD_LINUX || OSD_OS==OSD_HPUX || OSD_OSTYPE==OSD_WIN32 || OSD_OS==OSD_SCO || OSD_OS==OSD_FBSD
	hostent_ptr = gethostbyaddr((char *)&address, sizeof(struct in_addr), AF_INET);
#elif OSD_OS==OSD_NECEWS 
	hostent_ptr = gethostbyaddr((char *)&address, 4, AF_INET);
#else
	hostent_ptr = gethostbyaddr((void *)&address, sizeof(struct in_addr), AF_INET);
#endif

	if (NULL == hostent_ptr)
	{
		strncpy(hostname, "_unknown_", 10);
		return FALSE;
	}
	else
	{
		strncpy(hostname, (char *) hostent_ptr->h_name, hostname_length);
		return TRUE;
	}
}


/******************************************************************************
void	nlUnaliasHostname(char *hostname, int hostname_length)

Convert 'hostname' to the unaliased version.

keywords: blackbox, c, sysinfo, system, hostname
******************************************************************************/
/*PROTOTYPE*/
#if OSD_COMPILER==OSD_KR
void nlUnaliasHostname(hostname, hostname_length)
	char *hostname;
	int hostname_length;
#else
void nlUnaliasHostname(char *hostname, int hostname_length)
#endif
{
	struct hostent *hostent_ptr;
	char string[1000];

	hostent_ptr = gethostbyname(hostname);
	if (NULL == hostent_ptr)
	{
		sprintf(string, "could not get address for hostname <%s>", hostname);
		NL_PRINT(NL_OS, NL_ERROR, string);
		return;
	}

	strncpy(hostname, (char*) (hostent_ptr->h_name), hostname_length);

	return;
}


/******************************************************************************
NL_HOST_INET_ADDR	nlGetServerAddress(char *nodename)

	can take as an argument either a hostname or an internet address
		in dot notation

keywords: blackbox, c, sysinfo, system, address

******************************************************************************/
/*PROTOTYPE*/
#if OSD_COMPILER==OSD_KR
NL_HOST_INET_ADDR nlGetServerAddress(nodename)
	const char *nodename;
#else
NL_HOST_INET_ADDR nlGetServerAddress(const char *nodename)
#endif
{
	struct hostent *hostent_ptr;
	NL_HOST_INET_ADDR address = NULL;

#if OSD_OSTYPE==OSD_WIN32 /* begin NT-Specific section */
	address = inet_addr(nodename);
	if (INADDR_NONE == address)
	{
#endif

		hostent_ptr = gethostbyname(nodename);
		if (NULL == hostent_ptr)
		{
			if (!(address = nlStringToInetAddr(nodename)))
			{
				return (NL_HOST_INET_ADDR) NULL;
			}
		}
		else
		{
			address = (* (NL_HOST_INET_ADDR *) hostent_ptr->h_addr_list[0]);
		}

#if OSD_OSTYPE==OSD_WIN32 /* begin NT-Specific section */
	}
#endif

	return address;
}


/******************************************************************************
NL_HOST_INET_ADDR	nlGetHostAddress()

Get the internet address of the host.

keywords: blackbox, c, sysinfo, system, address

******************************************************************************/
/*PROTOTYPE*/
#if OSD_COMPILER==OSD_KR
NL_HOST_INET_ADDR nlGetHostAddress()
#else
NL_HOST_INET_ADDR nlGetHostAddress(void)
#endif
{
	static NL_HOST_INET_ADDR buffer_address = 0;

	wbaLog("NL LOG", "call to nlGetHostAddress()");
	if (buffer_address)
	{
		return buffer_address;
	}

#if OSD_OSTYPE==OSD_UNIX /* begin Unix-specific section */
	struct hostent *hostent_ptr;
	struct utsname hosttmp;
	int n;
	char string[1000];

	n = uname(&hosttmp);
	if (0 > n)
	{
		sprintf(string, "could not find hostname <%s>", hosttmp.nodename);
		NL_PRINT(NL_OS, NL_ERROR, string);
		return 0;
	}

	hostent_ptr = gethostbyname((char*) (hosttmp.nodename));

	if (NULL == hostent_ptr)
	{
		sprintf(string, "could not get address for hostname <%s>",
				hosttmp.nodename);
		NL_PRINT(NL_OS, NL_ERROR, string);
		return 0;
	}

	buffer_address = (* (NL_HOST_INET_ADDR *)hostent_ptr->h_addr_list[0]);

	return buffer_address;
#elif OSD_OSTYPE==OSD_WIN32 /* begin NT-Specific section */
	char hostname[NL_MAX_HOSTNAME_LEN];
	struct hostent *hostent_ptr;
	char string[1000];

	if (-1 == gethostname(hostname, NL_MAX_HOSTNAME_LEN))
	{
		sprintf(string, "could not find hostname <%s>", hostname);
		NL_PRINT(NL_OS, NL_ERROR, string);
		return 0;
	}

	hostent_ptr = gethostbyname(hostname);
	if (NULL == hostent_ptr)
	{
		sprintf(string, "could not get address for hostname <%s>", hostname);
		NL_PRINT(NL_OS, NL_ERROR, string);
		return 0;
	}

	buffer_address = (* (NL_HOST_INET_ADDR *)hostent_ptr->h_addr_list[0]);
	
	return buffer_address;
#endif
}

/******************************************************************************
long nlGetHostName(char *hostname, long hostname_length)

Get the internet name of the host.

keywords: blackbox, c, sysinfo, system, address

******************************************************************************/
/*PROTOTYPE*/
#if OSD_COMPILER==OSD_KR
long nlGetHostName(hostname, hostname_length)
	char *hostname;
	long hostname_length;
#else
long nlGetHostName(char *hostname, long hostname_length)
#endif
{
	wbaLog("NL LOG", "call to nlGetHostName()");
#if OSD_OSTYPE==OSD_UNIX /* begin Unix-specific section */
	struct utsname hosttmp;
	int n;
	char string[1000];

	n = uname(&hosttmp);
	if (0 > n)
	{
		sprintf(string, "could not find hostname");
		NL_PRINT(NL_OS, NL_ERROR, string);
		return FALSE;
	}

	strncpy(hostname, hosttmp.nodename, (unsigned int) hostname_length);

	return TRUE;

#elif OSD_OSTYPE==OSD_WIN32 /* begin NT-Specific section */
	char hostname_tmp[NL_MAX_HOSTNAME_LEN];
	char string[1000];

	if (-1 == gethostname(hostname_tmp, NL_MAX_HOSTNAME_LEN))
	{
		sprintf(string, "could not find hostname");
		NL_PRINT(NL_OS, NL_ERROR, string);
		return FALSE;
	}

	strncpy(hostname, hostname_tmp, (unsigned int) hostname_length);

	return TRUE;
#endif
}

#if OSD_OSTYPE==OSD_UNIX  /* UNIX Specific */
/******************************************************************************
int		nlIoctl(SOCKET_HANDLE_T fildes,long request,char *arg)

ioctl mapped to IOCTL.
******************************************************************************/
/*PROTOTYPE*/
#if OSD_COMPILER==OSD_KR
int nlIoctl(fildes, request, arg)
	SOCKET_HANDLE_T fildes;
	long request;
	char *arg;
#else
int nlIoctl(SOCKET_HANDLE_T fildes, long request, char *arg)
#endif
{
	return ioctl(fildes, (int) request, arg);
}

#elif OSD_OSTYPE==OSD_WIN32 /* NT Specific */
/******************************************************************************
int		nlIoctl(SOCKET_HANDLE_T s,long cmd,char *arg)

******************************************************************************/
/*PROTOTYPE*/
#if OSD_COMPILER==OSD_KR
int nlIoctl(s, cmd, arg)
	SOCKET_HANDLE_T s;
	long cmd;
	char *arg;
#else
int nlIoctl(SOCKET_HANDLE_T s, long cmd, char *arg)
#endif
{
	u_long argp = *arg;

	return ioctlsocket(s, cmd, &argp);
}


#endif

