#ifndef NL_SYSTEM_H
#define NL_SYSTEM_H

/******************************************************************************

Coral Tree nl/system.h
Copyright (C) 1998 Infinite Monkeys Incorporated

This program is free software; you can redistribute it and/or modify
it under the terms of the Artistic License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
Artistic License for more details.

You should have received a copy of the Artistic License along with
this program (see meta/ART_LICE); if not, write to coral@imonk.com.

******************************************************************************/

#if OSD_OSTYPE==OSD_UNIX  /* begin Unix-specific section */
// //#elif OSD_OSTYPE==OSD_UNIX  /* begin Unix-specific section */
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <pwd.h>
#include <sys/types.h>
#include <sys/socket.h>
/* THE netinet INCLUDES SEEM UNNECESSARY...AND THEY BREAK SGI */
#if 0 
#include <netinet/in.h>
#include <netinet/tcp.h>
#endif
#include <arpa/inet.h>
#include <sys/ioctl.h>
#include <sys/stat.h>
#include <sys/utsname.h>
#include <unistd.h>
#include <netdb.h>
#include <fcntl.h>
#include <errno.h>

/**** Solaris 2.x ***/
#if OSD_OS==OSD_SUNOS && OSD_OSVER==5
#include <sys/filio.h>
#endif

/**** NEC ***********/
#if OSD_OS==OSD_NECEWS && OSD_OSVER==9
#include <sys/filio.h>
#endif

/******* NT-specific *********/
// // #if OSD_OSTYPE==OSD_WIN32
#elif OSD_OSTYPE==OSD_WIN32
#include <windows.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys\types.h>
#include <stdarg.h>
#include <process.h>
#include <time.h>
#endif

typedef unsigned char           uint8;
typedef unsigned short          uint16;
typedef short                   int16;
typedef unsigned long           uint32;

/*--------------------------------------------------------------------------*/
/* OS dependent stuff                                                       */
/*--------------------------------------------------------------------------*/
#if OSD_OSTYPE==OSD_UNIX  /* begin Unix-specific section */
#define  NL_HOST_INET_ADDR	uint32
#define  HANDLETYPE             int
#define  SOCKET                 int
#define  TRUE                   1
#define  FALSE                  0
#define  SOCKET_ERROR           -1
#define  SOCKET_HANDLE_T        int
#define  INET_SOCK_ADDR_STRUCT  struct sockaddr_in
#if OSD_OS==OSD_HPUX && OSD_OSVER==10
#define  PTR_SOCK_ADDR_STRUCT   struct sockaddr_in *
#else
#define  PTR_SOCK_ADDR_STRUCT   struct sockaddr *
#endif
#define  INVALID_SOCKET         -1
#define  INVALID_SOCKET_HANDLE  -1
#define  ERRNO                  errno

#elif OSD_OSTYPE==OSD_WIN32 /* NT Specific */
#include <windows.h>
#define  HANDLETYPE             HANDLE
#define  SOCKET_HANDLE_T        SOCKET
#define  INET_SOCK_ADDR_STRUCT  SOCKADDR_IN
#define  PTR_SOCK_ADDR_STRUCT   LPSOCKADDR
#define  INVALID_SOCKET_HANDLE  INVALID_SOCKET
#define  ERRNO                  GetLastError()

#define  NL_HOST_INET_ADDR	uint32
#define  NL_HIGH_BYTE		1
#define  NL_LOW_BYTE		1

#endif

/******************************************************************************
MACROS
******************************************************************************/

#if OSD_OS==OSD_HPUX && OSD_OSVER==9
#define NL_SELECT(a,b,c,d,e)		select(a,(int *)b,(int *)c,(int *)d,e)
#else
#define NL_SELECT(a,b,c,d,e)		select(a,b,c,d,e)
#endif

#if OSD_OSTYPE==OSD_UNIX  /* begin Unix-specific section */
#define	 nlCloseSocket(fd);					close(fd);
#define  nlCloseHandle(fd);					close(fd);
#define  NL_IP_SOCKET_STARTUP      nlIpSocketStartup
#define  NL_IP_SOCKET_CLEANUP()      ;

#elif OSD_OSTYPE==OSD_WIN32 /* NT Specific */
#define  nlCloseSocket					closesocket
#define  nlCloseHandle					CloseHandle
#define  NL_IP_SOCKET_STARTUP                           nlIpSocketStartup
#define  NL_IP_SOCKET_CLEANUP                          nlIpSocketCleanup
#define  sleep(x)                                        Sleep(x*1000)
#endif

#endif /* NL_SYSTEM_H */
