
/******************************************************************************

Coral Tree dna/ippoint.cc
Copyright (C) 1998 Infinite Monkeys Incorporated

This program is free software; you can redistribute it and/or modify
it under the terms of the Artistic License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
Artistic License for more details.

You should have received a copy of the Artistic License along with
this program (see meta/ART_LICE); if not, write to coral@imonk.com.

******************************************************************************/

#include <gpl/gpl.h>
#include <dna/dna.h>

// Ensure that MAXHOSTNAMELEN is defined.
#ifndef	MAXHOSTNAMELEN
#define	MAXHOSTNAMELEN	256
#endif	/* MAXHOSTNAMELEN */

/***********************************************************************
GPL_Boolean dnaIpHostAddressPrint(const IP_HOST_ADDRESS address,
											GPL_String & string)
***********************************************************************/
GPL_Boolean dnaIpHostAddressPrint(const IP_HOST_ADDRESS address,
											GPL_String & string)
{
	string.Printf("%u.%u.%u.%u",
#if OSD_ENDIAN==OSD_LITTLE
		(address & 0x000000ff) >> 0,
		(address & 0x0000ff00) >> 8,
		(address & 0x00ff0000) >> 16,
		(address & 0xff000000) >> 24);
#else
		(address & 0xff000000) >> 24,
		(address & 0x00ff0000) >> 16,
		(address & 0x0000ff00) >> 8,
		(address & 0x000000ff) >> 0);
#endif
	return TRUE;
}

/***********************************************************************
GPL_Boolean dnaIpHostAddressParse(const GPL_String & string,
											IP_HOST_ADDRESS & address)
***********************************************************************/
GPL_Boolean dnaIpHostAddressParse(const GPL_String & string,
											IP_HOST_ADDRESS & address)
{
	unsigned long b1 = 0;
	unsigned long b2 = 0;
	unsigned long b3 = 0;
	unsigned long b4 = 0;
	address = 0;
	if (4 == sscanf(string.GetData(), "%u.%u.%u.%u", &b1, &b2, &b3, &b4))
	{
#if OSD_ENDIAN==OSD_LITTLE
		address = address | (b4 << 24);
		address = address | (b3 << 16);
		address = address | (b2 << 8);
		address = address | (b1 << 0);
#else
		address = address | (b1 << 24);
		address = address | (b2 << 16);
		address = address | (b3 << 8);
		address = address | (b4 << 0);
#endif
		return TRUE;
	}
	return FALSE;
}

/***********************************************************************
class DNA_IPPoint
***********************************************************************/
#define DNA_IPPOINT_COMPARE(x,y)	if((x) < (y))\
	{\
		return -1;\
	}\
	else if ((x) > (y))\
	{\
		return 1;\
	}

/***********************************************************************
DNA_IPPoint::DNA_IPPoint (void)
***********************************************************************/
DNA_IPPoint::DNA_IPPoint (void)
{
	osdMemset((void *) &addr, 0, sizeof(addr));
	addr.sin_family = DNA_IP_FAMILY;
	addr.sin_port = 0;
}

/***********************************************************************
DNA_IPPoint::~DNA_IPPoint (void)
***********************************************************************/
DNA_IPPoint::~DNA_IPPoint (void)
{
}

/***********************************************************************
GPL_String DNA_IPPoint::GetHost(void) const
***********************************************************************/
GPL_String DNA_IPPoint::GetHost(void) const
{
	GPL_String name;
	GetHost(addr.sin_addr.s_addr, name);
	return name;
}

/***********************************************************************
IP_PORT DNA_IPPoint::GetPort(void) const
***********************************************************************/
IP_PORT DNA_IPPoint::GetPort(void) const
{
	return ntohs(addr.sin_port);
}

/***********************************************************************
void DNA_IPPoint::SetHost(const IP_HOST_ADDRESS h)
***********************************************************************/
void DNA_IPPoint::SetHost(const IP_HOST_ADDRESS h)
{
	addr.sin_addr.s_addr = h;
}

/***********************************************************************
void DNA_IPPoint::SetHost(const GPL_String & host)
***********************************************************************/
void DNA_IPPoint::SetHost(const GPL_String & host)
{
	if (0 < host.GetLength())
	{
		IP_HOST_ADDRESS h;
		h = addr.sin_addr.s_addr;
		GetAddress(host, h);
		addr.sin_addr.s_addr = h;
	}
}

/***********************************************************************
void DNA_IPPoint::SetPort(const IP_PORT p)
***********************************************************************/
void DNA_IPPoint::SetPort(const IP_PORT p)
{
	addr.sin_port = htons(p);
}

/***********************************************************************
void DNA_IPPoint::SetPort(const GPL_String & service,
							const GPL_String & protocol)
***********************************************************************/
void DNA_IPPoint::SetPort(const GPL_String & service,
							const GPL_String & protocol)
{
	if (0 < service.GetLength() && 0 < protocol.GetLength())
	{
		GetPort(service, protocol, addr.sin_port);
	}
}

/***********************************************************************
long DNA_IPPoint::Compare(DNA_Point *p) const
***********************************************************************/
long DNA_IPPoint::Compare(DNA_Point *p) const
{
	if ((DNA_Point *) 0 == p || GetClassId() != p->GetClassId())
	{
		return -1;
	}

	DNA_IPPOINT_COMPARE(addr.sin_family, ((DNA_IPPoint *) p)->addr.sin_family);
	DNA_IPPOINT_COMPARE(addr.sin_port, ((DNA_IPPoint *) p)->addr.sin_port);
	DNA_IPPOINT_COMPARE(addr.sin_addr.s_addr, ((DNA_IPPoint *) p)->addr.sin_addr.s_addr);
	return 0;
}

/***********************************************************************
GPL_Boolean DNA_IPPoint::GetProtocol(const GPL_String & name,
									IP_PROTOCOL & protocol)
***********************************************************************/
GPL_Boolean DNA_IPPoint::GetProtocol(const GPL_String & name,
									IP_PROTOCOL & protocol)
{
	if (0 < name.GetLength())
	{
		register struct protoent *p =
			getprotobyname(name.GetData());
		if ((struct protoent *) 0 != p)
		{
			protocol = (IP_PROTOCOL) p->p_proto;
			return TRUE;
		}
	}
	return FALSE;
}

/***********************************************************************
GPL_Boolean DNA_IPPoint::GetPort(const GPL_String & service,
								const GPL_String & protocol, IP_PORT & port)
***********************************************************************/
GPL_Boolean DNA_IPPoint::GetPort(const GPL_String & service,
								const GPL_String & protocol, IP_PORT & port)
{
	if (0 < service.GetLength() && 0 < protocol.GetLength())
	{
		register struct servent *s =
			getservbyname(service.GetData(), protocol.GetData());
		if ((struct servent *) 0 != s)
		{
			port = ntohs((IP_PORT) s->s_port);
			return TRUE;
		}
	}
	return FALSE;
}

/***********************************************************************
GPL_Boolean DNA_IPPoint::GetPort(const GPL_String & name, IP_PORT & port)
***********************************************************************/
GPL_Boolean DNA_IPPoint::GetPort(const GPL_String & name, IP_PORT & port)
{
	port = 0;
	static GPL_RegularExpression numberExpression("^([0-9]+)$");
	if (numberExpression.Matches(name))
	{
		port = (IP_PORT) atoi(numberExpression.GetMatch(1));
	}
	else
	{
		DNA_IPPoint::GetPort(name, "tcp", port);
	}
	return 0 != port;
}

/***********************************************************************
GPL_Boolean DNA_IPPoint::GetService(const IP_PORT port,
									const GPL_String & protocol,
									GPL_String & service)
***********************************************************************/
GPL_Boolean DNA_IPPoint::GetService(const IP_PORT port,
									const GPL_String & protocol,
									GPL_String & service)
{
	if (0 < protocol.GetLength())
	{
		register struct servent *s = getservbyport(port, protocol.GetData());
		if ((struct servent *) 0 != s)
		{
			service = s->s_name;
			return TRUE;
		}
	}
	return FALSE;
}

/***********************************************************************
GPL_Boolean DNA_IPPoint::GetHost(GPL_String & host)
***********************************************************************/
GPL_Boolean DNA_IPPoint::GetHost(GPL_String & host)
{
#if OSD_OSTYPE==OSD_WIN32
	host.Resize(MAXHOSTNAMELEN);
	if (-1 == gethostname(host.GetData(), host.GetSize()))
	{
		wbaLog("DNA WARNING", "Could not find hostname");
		return FALSE;
	}
	return TRUE;
#else /* OSD_OSTYPE!=OSD_WIN32 */
	struct utsname uts;
	if (-1 == uname(&uts))
	{
		wbaLog("DNA WARNING", "Could not find hostname");
		return FALSE;
	}
	host = uts.nodename;
	return TRUE;
#endif
}

/***********************************************************************
GPL_Boolean DNA_IPPoint::GetHost(const IP_HOST_ADDRESS address,
								GPL_String & host)
***********************************************************************/
GPL_Boolean DNA_IPPoint::GetHost(const IP_HOST_ADDRESS address,
								GPL_String & host)
{
	int n = 4;
#if OSD_OS!=OSD_NECEWS
	n = sizeof(struct in_addr);
#endif /* OSD_OS!=OSD_NECEWS */
	register struct hostent *h =
		gethostbyaddr((const char *) &address, n, AF_INET);
	if ((struct hostent *) 0 != h)
	{
		host = h->h_name;
		return TRUE;
	}
	return FALSE;
}

/***********************************************************************
GPL_Boolean DNA_IPPoint::GetAddress(IP_HOST_ADDRESS & address)
***********************************************************************/
GPL_Boolean DNA_IPPoint::GetAddress(IP_HOST_ADDRESS & address)
{
	GPL_String host;
	if (GetHost(host))
	{
		return GetAddress(host, address);
	}
	return FALSE;
}

/***********************************************************************
GPL_Boolean DNA_IPPoint::GetAddress(const GPL_String & host,
								IP_HOST_ADDRESS & address)
***********************************************************************/
GPL_Boolean DNA_IPPoint::GetAddress(const GPL_String & host,
								IP_HOST_ADDRESS & address)
{
	if (0 == host.GetLength())
	{
		return FALSE;
	}

	// If host is in dotted notation...
	address = inet_addr(host.GetData());
#if OSD_OSTYPE==OSD_WIN32
	if (INADDR_NONE != address)
#else /* OSD_OSTYPE==OSD_WIN32 */
	if ((IP_HOST_ADDRESS) -1 != address)
#endif
	{
			return TRUE;
	}

	// If host is *not* in dotted notation...
	register struct hostent *h = gethostbyname(host.GetData());
	if ((struct hostent *) 0 != h)
	{
		address = *((IP_HOST_ADDRESS *) (h->h_addr_list[0]));
		return TRUE;
	}

	// Try to parse the host name as dotted numbers, e.g., 192.168.0.95.
	// NOTE: This should be redundant with inet_addr(), but it is
	// left in anyway.
	if (dnaIpHostAddressParse(host, address))
	{
		return TRUE;
	}
	return FALSE;
}

/***********************************************************************
GPL_Boolean DNA_IPPoint::UnaliasHost(const GPL_String & alias, GPL_String & host)
***********************************************************************/
GPL_Boolean DNA_IPPoint::UnaliasHost(const GPL_String & alias, GPL_String & host)
{
	if (0 < alias.GetLength())
	{
		register struct hostent *h = gethostbyname(alias.GetData());
		if ((struct hostent *) 0 != h)
		{
			host = h->h_name;
			return TRUE;
		}
		GPL_String string;
		string.Printf("Could not get address for hostname <%s>", alias.GetData());
		wbaLog("DNA WARNING", string);
	}
	return TRUE;
}

/***********************************************************************
GPL_Boolean DNA_IPPoint::Print(GPL_String & string,
								const GPL_Verbosity verbosity) const
***********************************************************************/
GPL_Boolean DNA_IPPoint::Print(GPL_String & string,
								const GPL_Verbosity verbosity) const
{
	GPL_String address = "x.x.x.x";
	dnaIpHostAddressPrint(addr.sin_addr.s_addr, address);
	GPL_String image;
	image.Printf("%d,%d,%s", addr.sin_family, addr.sin_port, address.GetData());
	string.Append(image);
	return TRUE;
}

/***********************************************************************
GPL_Boolean DNA_IPPoint::Parse(const GPL_String & string)
***********************************************************************/
GPL_Boolean DNA_IPPoint::Parse(const GPL_String & string)
{
	static GPL_RegularExpression expression
		("([0-9]+),([0-9]+),([0-9]+\\.[0-9]+\\.[0-9]+\\.[0-9]+)");
	if (expression.Matches(string))
	{
		// Bogus family values are silently ignored.
		addr.sin_family = DNA_IP_FAMILY;
		addr.sin_port = atoi(expression.GetMatch(2).GetData());
		const GPL_String string = expression.GetMatch(3);
		IP_HOST_ADDRESS address = (IP_HOST_ADDRESS) 0;
		if (dnaIpHostAddressParse(string, address))
		{
			addr.sin_addr.s_addr = address;
			return TRUE;
		}
	}
	return FALSE;
}

GPL_OBJECT_IMPLEMENTATION(DNA_IPPoint, 0x40070002, "DNA_IPPoint");

