#ifndef DNA_IPPOINT_H
#define DNA_IPPOINT_H

/******************************************************************************

Coral Tree dna/ippoint.h
Copyright (C) 1998 Infinite Monkeys Incorporated

This program is free software; you can redistribute it and/or modify
it under the terms of the Artistic License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
Artistic License for more details.

You should have received a copy of the Artistic License along with
this program (see meta/ART_LICE); if not, write to coral@imonk.com.

******************************************************************************/

#include <gpl/gpl.h>

#include <dna/binding.h>

#if OSD_OSTYPE==OSD_UNIX

#define	SOCKET_HANDLE				int
#define	INVALID_SOCKET_HANDLE		-1
#define	IP_HOST_ADDRESS				unsigned long
#define	IP_SOCKET_ADDRESS			struct sockaddr_in
#define	PTR_IP_SOCKET_ADDRESS		struct sockaddr *
#define	IP_PORT						unsigned short
#define	IP_PROTOCOL					int
#define	DNA_EWOULDBLOCK				EWOULDBLOCK
#define	DNA_BEGIN_EINTR_LOOP			do{
#define	DNA_END_EINTR_LOOP(if_error)	} while ((if_error) && (OSD_ERRNO == EINTR))

#elif OSD_OSTYPE==OSD_WIN32

#define	SOCKET_HANDLE				SOCKET
#define	INVALID_SOCKET_HANDLE		INVALID_SOCKET
#define	IP_HOST_ADDRESS				unsigned long
#define	IP_SOCKET_ADDRESS			SOCKADDR_IN
#define	PTR_IP_SOCKET_ADDRESS		LPSOCKADDR
#define	IP_PORT						unsigned short
#define	IP_PROTOCOL					int
#define	DNA_WINSOCK_VERSION_HIGH_BYTE	1
#define	DNA_WINSOCK_VERSION_LOW_BYTE		1
// The following was deactivated in the MS-supplied winsock.h;
// alternately, one could reactivate that area of the header.
#define	DNA_EWOULDBLOCK				WSAEWOULDBLOCK
#define	DNA_BEGIN_EINTR_LOOP			do{
#define	DNA_END_EINTR_LOOP(if_error)	} while ((if_error) && (OSD_ERRNO == WSAEINTR))
#define	sleep(x)					Sleep(x*1000)

#endif /* OSD_OSTYPE==OSD_WIN32 */

#if OSD_OS==OSD_HPUX && OSD_OSVER==9
#define DNA_SELECT(a,b,c,d,e)		select(a,(int *)b,(int *)c,(int *)d,e)
#else /* OSD_OS!=OSD_HPUX || OSD_OSVER!=9 */
#define DNA_SELECT(a,b,c,d,e)		select(a,b,c,d,e)
#endif /* OSD_OS!=OSD_HPUX || OSD_OSVER!=9 */

// This is the family for all IP bindings and connections
// whose value appears in DNA_IPPoint::addr.sin_family.
#define	DNA_IP_FAMILY	AF_INET

extern GPL_Boolean dnaIpHostAddressPrint(const IP_HOST_ADDRESS address,
											GPL_String & string);
extern GPL_Boolean dnaIpHostAddressParse(const GPL_String & string,
											IP_HOST_ADDRESS & address);

/******************************************************************************
class DNA_IPPoint

Internet Protocol peer identification.
******************************************************************************/
class DNA_IPPoint : public DNA_Point
{
	public:
		DNA_IPPoint (void);
		virtual ~DNA_IPPoint (void);

		const IP_SOCKET_ADDRESS & GetAddress(void) const { return addr; }
		virtual GPL_String GetHost(void) const;
		virtual IP_PORT GetPort(void) const;

		void SetHost(const IP_HOST_ADDRESS h);
		void SetHost(const GPL_String & host);
		void SetPort(const IP_PORT p);
		void SetPort(const GPL_String & service, const GPL_String & protocol);

		virtual long Compare(DNA_Point *p) const;

		static GPL_Boolean GetProtocol(const GPL_String & name,
										IP_PROTOCOL & protocol);
		static GPL_Boolean GetPort(const GPL_String & service,
									const GPL_String & protocol,
									IP_PORT & port);
		static GPL_Boolean GetPort(const GPL_String & name, IP_PORT & port);
		static GPL_Boolean GetService(const IP_PORT port,
										const GPL_String & protocol,
										GPL_String & service);
		static GPL_Boolean GetHost(GPL_String & host);
		static GPL_Boolean GetHost(const IP_HOST_ADDRESS address,
									GPL_String & host);
		static GPL_Boolean GetAddress(IP_HOST_ADDRESS & address);
		static GPL_Boolean GetAddress(const GPL_String & host,
										IP_HOST_ADDRESS & address);
		static GPL_Boolean UnaliasHost(const GPL_String & alias,
										GPL_String & host);

		// GPL_Depictable
		virtual GPL_Boolean Print(GPL_String & string,
									const GPL_Verbosity verbosity) const;
		virtual GPL_Boolean Parse(const GPL_String & string);

		// GPL_Object
		GPL_OBJECT_INTERFACE(DNA_IPPoint);

	protected:
		IP_SOCKET_ADDRESS addr;
};

#endif /* DNA_IPPOINT_H */
